@@ -1,4 +1,4 @@
-#!/usr/local/bin/node
+#!/usr/bin/env node
 
 'use strict';
 
@@ -44,7 +44,7 @@ var noArgs = function (fn) {
 
 var identity = function (i) { return i; };
 
-// like Q.all, but runs the comands in series
+// like Q.all, but runs the commands in series
 // useful for ensuring env state (like which branch is checked out)
 var allInSeries = function (fn) {
   return function (args) {
@@ -103,10 +103,10 @@ then(function (tags) {
     sort(semver.rcompare);
 }).
 then(function (tags) {
-  var major = tags[0].split('.')[0] + '.x';
+  var major = tags[0].split('.')[0];
   return tags.
     filter(function (ver) {
-      return semver.satisfies(ver, major);
+      return semver(ver).major == major;
     });
 }).
 then(function (tags) {
@@ -123,9 +123,12 @@ then(function (tags) {
     value();
 }).
 then(function (tags) {
+  var master = tags.pop();
+  var stable = tags.pop();
+
   return [
-    { name: 'v1.0.x', tag: tags[0] },
-    { name: 'master', tag: tags[1] }
+    { name: stable.replace(/\d+$/, 'x'), tag: stable },
+    { name: 'master', tag: master}
   ];
 }).
 then(allInSeries(function (branch) {
@@ -142,7 +145,7 @@ then(allInSeries(function (branch) {
         line = line.split(' ');
         var sha = line.shift();
         var msg = line.join(' ');
-        return sha + (msg.toLowerCase().indexOf('fix') === -1 ? '   ' : ' * ') + msg;
+        return sha + ((/fix\([^\)]+\):/i.test(msg))  ? ' * ' : '   ') + msg;
       });
       branch.log = log.map(function (line) {
         return line.substr(41);
