@@ -50,10 +50,10 @@ function extractNumberSymbols(content, localeInfo, currencySymbols) {
 function extractCurrencySymbols(content) {
   //eval script in the current context so that we get access to all the symbols
   eval(content.toString());
-  var currencySymbols = goog.i18n.currency.CurrencyInfo;
-  currencySymbols.__proto__ = goog.i18n.currency.CurrencyInfoTier2;
+  // var currencySymbols = goog.i18n.currency.CurrencyInfo;
+  // currencySymbols.__proto__ = goog.i18n.currency.CurrencyInfoTier2;
 
-  return currencySymbols;
+  return Object.assign({}, goog.i18n.currency.CurrencyInfoTier2, goog.i18n.currency.CurrencyInfo);
 }
 
 function extractDateTimeSymbols(content, localeInfo) {
@@ -79,7 +79,7 @@ function pluralExtractor(content, localeInfo) {
     goog.LOCALE = localeIds[i].match(/[^_]+/)[0];
     try {
       eval(contentText);
-    } catch(e) {
+    } catch (e) {
       console.log("Error in eval(contentText): " + e.stack);
     }
     if (!goog.i18n.pluralRules.select) {
@@ -87,6 +87,7 @@ function pluralExtractor(content, localeInfo) {
       continue;
     }
     var temp = goog.i18n.pluralRules.select.toString().
+        replace(/function\s+\(/g, 'function(').
         replace(/goog\.i18n\.pluralRules\.Keyword/g, 'PLURAL_CATEGORY').
         replace(/goog\.i18n\.pluralRules\.get_vf_/g, 'getVF').
         replace(/goog\.i18n\.pluralRules\.get_wt_/g, 'getWT').
@@ -132,7 +133,7 @@ function canonicalizeForJsonStringify(unused_key, object) {
 
 function serializeContent(localeObj) {
   return JSON.stringify(localeObj, canonicalizeForJsonStringify, '  ')
-    .replace(new RegExp('[\\u007f-\\uffff]', 'g'), function(c) { return '\\u'+('0000'+c.charCodeAt(0).toString(16)).slice(-4); })
+    .replace(new RegExp('[\\u007f-\\uffff]', 'g'), function(c) { return '\\u' + ('0000' + c.charCodeAt(0).toString(16)).slice(-4); })
     .replace(/"@@|@@"/g, '');
 }
 
@@ -160,6 +161,7 @@ function outputLocale(localeInfo, localeID) {
   if (!localeObj.DATETIME_FORMATS) {
     localeObj.DATETIME_FORMATS = fallBackObj.DATETIME_FORMATS;
   }
+  localeObj.localeID = localeID;
   localeObj.id = correctedLocaleId(localeID);
 
   var getDecimals = [
@@ -200,10 +202,11 @@ function outputLocale(localeInfo, localeID) {
     DATETIME_FORMATS: localeObj.DATETIME_FORMATS,
     NUMBER_FORMATS: localeObj.NUMBER_FORMATS,
     pluralCat: localeObj.pluralCat,
-    id: localeObj.id
+    id: localeObj.id,
+    localeID: localeID
   };
 
-  var content = serializeContent(localeInfo[localeID]);
+  var content = serializeContent(localeObj);
   if (content.indexOf('getVF(') < 0) {
     getVF = '';
   }
@@ -223,7 +226,7 @@ function outputLocale(localeInfo, localeID) {
           getDecimals + getVF + getWT +
           '$provide.value("$locale", ';
 
-  var suffix = ');\n}]);';
+  var suffix = ');\n}]);\n';
 
   return prefix + content + suffix;
 }
