@@ -42,8 +42,9 @@ angular.scenario.dsl = angular.scenario.dsl || function(name, fn) {
     /* jshint -W040 *//* The dsl binds `this` for us when calling chained functions */
     function executeStatement(statement, args) {
       var result = statement.apply(this, args);
-      if (angular.isFunction(result) || result instanceof angular.scenario.Future)
+      if (angular.isFunction(result) || result instanceof angular.scenario.Future) {
         return result;
+      }
       var self = this;
       var chain = angular.extend({}, result);
       angular.forEach(chain, function(value, name) {
@@ -103,7 +104,7 @@ angular.scenario.matcher = angular.scenario.matcher || function(name, fn) {
  */
 angular.scenario.setUpAndRun = function(config) {
   var href = window.location.href;
-  var body = _jQuery(document.body);
+  var body = _jQuery(window.document.body);
   var output = [];
   var objModel = new angular.scenario.ObjectModel($runner);
 
@@ -112,7 +113,7 @@ angular.scenario.setUpAndRun = function(config) {
   }
 
   angular.forEach(angular.scenario.output, function(fn, name) {
-    if (!output.length || indexOf(output,name) != -1) {
+    if (!output.length || output.indexOf(name) != -1) {
       var context = body.append('<div></div>').find('div:last');
       context.attr('id', name);
       fn.call({}, context, $runner, objModel);
@@ -217,10 +218,10 @@ function callerFile(offset) {
     if (line) {
       if (line.indexOf('@') !== -1) {
         // Firefox
-        line = line.substring(line.indexOf('@')+1);
+        line = line.substring(line.indexOf('@') + 1);
       } else {
         // Chrome
-        line = line.substring(line.indexOf('(')+1).replace(')', '');
+        line = line.substring(line.indexOf('(') + 1).replace(')', '');
       }
     }
 
@@ -238,7 +239,7 @@ function callerFile(offset) {
  *
  * To work around this we instead use our own handler that fires a real event.
  */
-(function(fn){
+(function(fn) {
   // We need a handle to the original trigger function for input tests.
   var parentTrigger = fn._originalTrigger = fn.trigger;
   fn.trigger = function(type) {
@@ -268,7 +269,7 @@ _jQuery.fn.bindings = function(windowJquery, bindExp) {
       bindSelector = '.ng-binding:visible';
   if (angular.isString(bindExp)) {
     bindExp = bindExp.replace(/\s/g, '');
-    match = function (actualExp) {
+    match = function(actualExp) {
       if (actualExp) {
         actualExp = actualExp.replace(/\s/g, '');
         if (actualExp == bindExp) return true;
@@ -292,7 +293,7 @@ _jQuery.fn.bindings = function(windowJquery, bindExp) {
   }
 
   function push(value) {
-    if (value === undefined) {
+    if (angular.isUndefined(value)) {
       value = '';
     } else if (typeof value !== 'string') {
       value = angular.toJson(value);
@@ -302,27 +303,21 @@ _jQuery.fn.bindings = function(windowJquery, bindExp) {
 
   selection.each(function() {
     var element = windowJquery(this),
-        binding;
-    if (binding = element.data('$binding')) {
-      if (typeof binding == 'string') {
-        if (match(binding)) {
-          push(element.scope().$eval(binding));
+        bindings;
+    if (bindings = element.data('$binding')) {
+      for (var expressions = [], binding, j=0, jj=bindings.length; j < jj; j++) {
+        binding = bindings[j];
+
+        if (binding.expressions) {
+          expressions = binding.expressions;
+        } else {
+          expressions = [binding];
         }
-      } else {
-        if (!angular.isArray(binding)) {
-          binding = [binding];
-        }
-        for(var fns, j=0, jj=binding.length;  j<jj; j++) {
-          fns = binding[j];
-          if (fns.parts) {
-            fns = fns.parts;
-          } else {
-            fns = [fns];
-          }
-          for (var scope, fn, i = 0, ii = fns.length; i < ii; i++) {
-            if(match((fn = fns[i]).exp)) {
-              push(fn(scope = scope || element.scope()));
-            }
+        for (var scope, expression, i = 0, ii = expressions.length; i < ii; i++) {
+          expression = expressions[i];
+          if (match(expression)) {
+            scope = scope || element.scope();
+            push(scope.$eval(expression));
           }
         }
       }
