@@ -1,17 +1,21 @@
 'use strict';
 
+/* globals support: false */
+
 describe('injector', function() {
   var providers;
   var injector;
   var providerInjector;
+  var controllerProvider;
 
-  beforeEach(module(function($provide, $injector) {
+  beforeEach(module(function($provide, $injector, $controllerProvider) {
     providers = function(name, factory, annotations) {
-      $provide.factory(name, extend(factory, annotations||{}));
+      $provide.factory(name, extend(factory, annotations || {}));
     };
     providerInjector = $injector;
+    controllerProvider = $controllerProvider;
   }));
-  beforeEach(inject(function($injector){
+  beforeEach(inject(function($injector) {
     injector = $injector;
   }));
 
@@ -28,11 +32,17 @@ describe('injector', function() {
 
   it('should inject providers', function() {
     providers('a', function() {return 'Mi';});
-    providers('b', function(mi) {return mi+'sko';}, {$inject:['a']});
+    providers('b', function(mi) {return mi + 'sko';}, {$inject:['a']});
     expect(injector.get('b')).toEqual('Misko');
   });
 
 
+  it('should check its modulesToLoad argument', function() {
+    expect(function() { angular.injector('test'); })
+        .toThrowMinErr('ng', 'areq');
+  });
+
+
   it('should resolve dependency graph and instantiate all services just once', function() {
     var log = [];
 
@@ -45,12 +55,12 @@ describe('injector', function() {
 //   s6
 
 
-    providers('s1', function() { log.push('s1'); }, {$inject: ['s2', 's5', 's6']});
-    providers('s2', function() { log.push('s2'); }, {$inject: ['s3', 's4', 's5']});
-    providers('s3', function() { log.push('s3'); }, {$inject: ['s6']});
-    providers('s4', function() { log.push('s4'); }, {$inject: ['s3', 's5']});
-    providers('s5', function() { log.push('s5'); });
-    providers('s6', function() { log.push('s6'); });
+    providers('s1', function() { log.push('s1'); return {}; }, {$inject: ['s2', 's5', 's6']});
+    providers('s2', function() { log.push('s2'); return {}; }, {$inject: ['s3', 's4', 's5']});
+    providers('s3', function() { log.push('s3'); return {}; }, {$inject: ['s6']});
+    providers('s4', function() { log.push('s4'); return {}; }, {$inject: ['s3', 's5']});
+    providers('s5', function() { log.push('s5'); return {}; });
+    providers('s6', function() { log.push('s6'); return {}; });
 
     injector.get('s1');
 
@@ -59,7 +69,7 @@ describe('injector', function() {
 
 
   it('should allow query names', function() {
-    providers('abc', function () { return ''; });
+    providers('abc', function() { return ''; });
 
     expect(injector.has('abc')).toBe(true);
     expect(injector.has('xyz')).toBe(false);
@@ -74,6 +84,22 @@ describe('injector', function() {
   });
 
 
+  it('should provide the caller name if given', function() {
+    expect(function() {
+      injector.get('idontexist', 'callerName');
+    }).toThrowMinErr("$injector", "unpr", "Unknown provider: idontexistProvider <- idontexist <- callerName");
+  });
+
+
+  it('should provide the caller name for controllers', function() {
+    controllerProvider.register('myCtrl', function(idontexist) {});
+    var $controller = injector.get('$controller');
+    expect(function() {
+      $controller('myCtrl', {$scope: {}});
+    }).toThrowMinErr("$injector", "unpr", "Unknown provider: idontexistProvider <- idontexist <- myCtrl");
+  });
+
+
   it('should not corrupt the cache when an object fails to get instantiated', function() {
     expect(function() {
       injector.get('idontexist');
@@ -98,6 +124,10 @@ describe('injector', function() {
     expect($injector).not.toBe(providerInjector);
   }));
 
+  it('should have an false strictDi property', inject(function($injector) {
+    expect($injector.strictDi).toBe(false);
+  }));
+
 
   describe('invoke', function() {
     var args;
@@ -154,24 +184,28 @@ describe('injector', function() {
       fn.$inject = ['a'];
       expect(annotate(fn)).toBe(fn.$inject);
       expect(annotate(function() {})).toEqual([]);
-      expect(annotate(function () {})).toEqual([]);
+      expect(annotate(function() {})).toEqual([]);
+      // jscs:disable disallowSpacesInAnonymousFunctionExpression
       expect(annotate(function  () {})).toEqual([]);
       expect(annotate(function /* */ () {})).toEqual([]);
+      // jscs:enable disallowSpacesInAnonymousFunctionExpression
     });
 
 
     it('should create $inject', function() {
       var extraParans = angular.noop;
+      // jscs:disable disallowSpacesInFunctionDeclaration
       // keep the multi-line to make sure we can handle it
       function $f_n0 /*
           */(
           $a, // x, <-- looks like an arg but it is a comment
           b_, /* z, <-- looks like an arg but it is a
                  multi-line comment
-                 function (a, b) {}
+                 function(a, b) {}
                  */
           _c,
           /* {some type} */ d) { extraParans();}
+      // jscs:enable disallowSpacesInFunctionDeclaration
       expect(annotate($f_n0)).toEqual(['$a', 'b_', '_c',  'd']);
       expect($f_n0.$inject).toEqual(['$a', 'b_', '_c',  'd']);
     });
@@ -182,6 +216,10 @@ describe('injector', function() {
       expect(annotate(beforeEachFn)).toEqual(['foo']);
     });
 
+    it('should not strip service names with a single underscore', function() {
+      function beforeEachFn(_) { /* _ = _ */ }
+      expect(annotate(beforeEachFn)).toEqual(['_']);
+    });
 
     it('should handle no arg functions', function() {
       function $f_n0() {}
@@ -191,7 +229,7 @@ describe('injector', function() {
 
 
     it('should handle no arg functions with spaces in the arguments list', function() {
-      function fn( ) {}
+      function fn() {}
       expect(annotate(fn)).toEqual([]);
       expect(fn.$inject).toEqual([]);
     });
@@ -203,6 +241,12 @@ describe('injector', function() {
       expect($f_n0.$inject).toEqual(['$a_']);
     });
 
+    it('should handle functions with overridden toString', function() {
+      function fn(a) {}
+      fn.toString = function() { return 'fn'; };
+      expect(annotate(fn)).toEqual(['a']);
+      expect(fn.$inject).toEqual(['a']);
+    });
 
     it('should throw on non function arg', function() {
       expect(function() {
@@ -211,8 +255,55 @@ describe('injector', function() {
     });
 
 
+    describe('es6', function() {
+      /*jshint -W061 */
+      if (support.ES6Function) {
+        // The functions are generated using `eval` as just having the ES6 syntax can break some browsers.
+        it('should be possible to annotate functions that are declared using ES6 syntax', function() {
+          expect(annotate(eval('({ fn(x) { return; } })').fn)).toEqual(['x']);
+        });
+      }
+
+
+      if (support.fatArrow) {
+        it('should create $inject for arrow functions', function() {
+          expect(annotate(eval('(a, b) => a'))).toEqual(['a', 'b']);
+        });
+      }
+
+
+      if (support.fatArrow) {
+        it('should create $inject for arrow functions with no parenthesis', function() {
+          expect(annotate(eval('a => a'))).toEqual(['a']);
+        });
+      }
+
+
+      if (support.fatArrow) {
+        it('should take args before first arrow', function() {
+          expect(annotate(eval('a => b => b'))).toEqual(['a']);
+        });
+      }
+
+      if (support.classes) {
+        it('should be possible to instantiate ES6 classes', function() {
+          providers('a', function() { return 'a-value'; });
+          var Clazz = eval('(class { constructor(a) { this.a = a; } aVal() { return this.a; } })');
+          var instance = injector.instantiate(Clazz);
+          expect(instance).toEqual(new Clazz('a-value'));
+          expect(instance.aVal()).toEqual('a-value');
+        });
+      }
+      /*jshint +W061 */
+    });
+
+
     it('should publish annotate API', function() {
-      expect(injector.annotate).toBe(annotate);
+      expect(angular.mock.$$annotate).toBe(annotate);
+      spyOn(angular.mock, '$$annotate').and.callThrough();
+      function fn() {}
+      injector.annotate(fn);
+      expect(angular.mock.$$annotate).toHaveBeenCalledWith(fn);
     });
   });
 
@@ -242,9 +333,10 @@ describe('injector', function() {
 
   describe('module', function() {
     it('should provide $injector even when no module is requested', function() {
-      var $provide, $injector = createInjector([
-        angular.extend(function(p) { $provide = p; }, {$inject: ['$provide']})
-      ]);
+      var $provide,
+        $injector = createInjector([
+          angular.extend(function(p) { $provide = p; }, {$inject: ['$provide']})
+        ]);
       expect($injector.get('$injector')).toBe($injector);
     });
 
@@ -259,7 +351,7 @@ describe('injector', function() {
           $provide.value('a', a);
         },
         angular.extend(function(p, serviceA) {
-          p.value('b', serviceA.$get() + 'B' );
+          p.value('b', serviceA.$get() + 'B');
         }, {$inject:['$provide', 'aProvider']}),
         ['$provide', 'bProvider', function(p, serviceB) {
           p.value('c', serviceB.$get() + 'C');
@@ -288,16 +380,16 @@ describe('injector', function() {
 
     it('should load dependant modules only once', function() {
       var log = '';
-      angular.module('a', [], function(){ log += 'a'; });
-      angular.module('b', ['a'], function(){ log += 'b'; });
-      angular.module('c', ['a', 'b'], function(){ log += 'c'; });
+      angular.module('a', [], function() { log += 'a'; });
+      angular.module('b', ['a'], function() { log += 'b'; });
+      angular.module('c', ['a', 'b'], function() { log += 'c'; });
       createInjector(['c', 'c']);
       expect(log).toEqual('abc');
     });
 
     it('should load different instances of dependent functions', function() {
       function  generateValueModule(name, value) {
-        return function ($provide) {
+        return function($provide) {
           $provide.value(name, value);
         };
       }
@@ -320,8 +412,8 @@ describe('injector', function() {
 
     it('should execute runBlocks after injector creation', function() {
       var log = '';
-      angular.module('a', [], function(){ log += 'a'; }).run(function() { log += 'A'; });
-      angular.module('b', ['a'], function(){ log += 'b'; }).run(function() { log += 'B'; });
+      angular.module('a', [], function() { log += 'a'; }).run(function() { log += 'A'; });
+      angular.module('b', ['a'], function() { log += 'b'; }).run(function() { log += 'B'; });
       createInjector([
         'b',
         valueFn(function() { log += 'C'; }),
@@ -330,6 +422,29 @@ describe('injector', function() {
       expect(log).toEqual('abABCD');
     });
 
+    it('should execute own config blocks after all own providers are invoked', function() {
+      var log = '';
+      angular.module('a', ['b'])
+      .config(function($aProvider) {
+        log += 'aConfig;';
+      })
+      .provider('$a', function() {
+        log += '$aProvider;';
+        this.$get = function() {};
+      });
+      angular.module('b', [])
+      .config(function($bProvider) {
+        log += 'bConfig;';
+      })
+      .provider('$b', function() {
+        log += '$bProvider;';
+        this.$get = function() {};
+      });
+
+      createInjector(['a']);
+      expect(log).toBe('$bProvider;bConfig;$aProvider;aConfig;');
+    });
+
     describe('$provide', function() {
 
       it('should throw an exception if we try to register a service called "hasOwnProperty"', function() {
@@ -353,7 +468,7 @@ describe('injector', function() {
         it('should create configuration injectable constants', function() {
           var log = [];
           createInjector([
-            function($provide){
+            function($provide) {
               $provide.constant('abc', 123);
               $provide.constant({a: 'A', b:'B'});
               return function(a) {
@@ -596,6 +711,23 @@ describe('injector', function() {
           expect(log.join('; ')).
             toBe('myDecoratedService:input,dependency1; myService:decInput; dec+origReturn');
         });
+
+
+        it('should allow for decorators to $injector', function() {
+          injector = createInjector(['ng', function($provide) {
+            $provide.decorator('$injector', function($delegate) {
+              return extend({}, $delegate, {get: function(val) {
+                if (val === 'key') {
+                  return 'value';
+                }
+                return $delegate.get(val);
+              }});
+            });
+          }]);
+
+          expect(injector.get('key')).toBe('value');
+          expect(injector.get('$http')).not.toBeUndefined();
+        });
       });
     });
 
@@ -622,7 +754,7 @@ describe('injector', function() {
       it('should decorate the missing service error with module name', function() {
         angular.module('TestModule', [], function(xyzzy) {});
         expect(function() {
-          createInjector(['TestModule' ]);
+          createInjector(['TestModule']);
         }).toThrowMinErr(
           '$injector', 'modulerr', /Failed to instantiate module TestModule due to:\n.*\[\$injector:unpr] Unknown provider: xyzzy/
         );
@@ -630,7 +762,7 @@ describe('injector', function() {
 
 
       it('should decorate the missing service error with module function', function() {
-        function myModule(xyzzy){}
+        function myModule(xyzzy) {}
         expect(function() {
           createInjector([myModule]);
         }).toThrowMinErr(
@@ -640,7 +772,7 @@ describe('injector', function() {
 
 
       it('should decorate the missing service error with module array function', function() {
-        function myModule(xyzzy){}
+        function myModule(xyzzy) {}
         expect(function() {
           createInjector([['xyzzy', myModule]]);
         }).toThrowMinErr(
@@ -651,8 +783,8 @@ describe('injector', function() {
 
       it('should throw error when trying to inject oneself', function() {
         expect(function() {
-          createInjector([function($provide){
-            $provide.factory('service', function(service){});
+          createInjector([function($provide) {
+            $provide.factory('service', function(service) {});
             return function(service) {};
           }]);
         }).toThrowMinErr('$injector', 'cdep', 'Circular dependency found: service <- service');
@@ -661,9 +793,9 @@ describe('injector', function() {
 
       it('should throw error when trying to inject circular dependency', function() {
         expect(function() {
-          createInjector([function($provide){
-            $provide.factory('a', function(b){});
-            $provide.factory('b', function(a){});
+          createInjector([function($provide) {
+            $provide.factory('a', function(b) {});
+            $provide.factory('b', function(a) {});
             return function(a) {};
           }]);
         }).toThrowMinErr('$injector', 'cdep', 'Circular dependency found: a <- b <- a');
@@ -678,12 +810,12 @@ describe('injector', function() {
     var Instance = function() { this.name = 'angular'; };
 
     function createInjectorWithValue(instanceName, instance) {
-      return createInjector([ ['$provide', function(provide) {
+      return createInjector([['$provide', function(provide) {
         provide.value(instanceName, instance);
       }]]);
     }
     function createInjectorWithFactory(serviceName, serviceDef) {
-      return createInjector([ ['$provide', function(provide) {
+      return createInjector([['$provide', function(provide) {
         provide.factory(serviceName, serviceDef);
       }]]);
     }
@@ -716,7 +848,7 @@ describe('injector', function() {
     var $injector;
 
     beforeEach(function() {
-      $injector = createInjector([ function($provide) {
+      $injector = createInjector([function($provide) {
         $provide.value('book', 'moby');
         $provide.value('author', 'melville');
       }]);
@@ -780,7 +912,7 @@ describe('injector', function() {
     var $injector;
 
     beforeEach(function() {
-      $injector = createInjector([ function($provide) {
+      $injector = createInjector([function($provide) {
         $provide.value('book', 'moby');
         $provide.value('author', 'melville');
       }]);
@@ -886,7 +1018,104 @@ describe('injector', function() {
         createInjector([function($provide) {
           $provide.value('name', 'angular');
         }, instanceLookupInModule]);
-      }).toThrowMatching(/\[\$injector:unpr] Unknown provider: name/);
+      }).toThrowError(/\[\$injector:unpr] Unknown provider: name/);
     });
   });
 });
+
+describe('strict-di injector', function() {
+  beforeEach(inject.strictDi(true));
+
+  describe('with ngMock', function() {
+    it('should not throw when calling mock.module() with "magic" annotations', function() {
+      expect(function() {
+        module(function($provide, $httpProvider, $compileProvider) {
+          // Don't throw!
+        });
+      }).not.toThrow();
+    });
+
+
+    it('should not throw when calling mock.inject() with "magic" annotations', function() {
+      expect(function() {
+        inject(function($rootScope, $compile, $http) {
+          // Don't throw!
+        });
+      }).not.toThrow();
+    });
+  });
+
+
+  it('should throw if magic annotation is used by service', function() {
+    module(function($provide) {
+      $provide.service({
+        '$test': function() { return this; },
+        '$test2': function($test) { return this; }
+      });
+    });
+    inject(function($injector) {
+      expect(function() {
+        $injector.invoke(function($test2) {});
+      }).toThrowMinErr('$injector', 'strictdi');
+    });
+  });
+
+
+  it('should throw if magic annotation is used by provider', function() {
+    module(function($provide) {
+      $provide.provider({
+        '$test': function() { this.$get = function($rootScope) { return $rootScope; }; }
+      });
+    });
+    inject(function($injector) {
+      expect(function() {
+        $injector.invoke(['$test', function($test) {}]);
+      }).toThrowMinErr('$injector', 'strictdi');
+    });
+  });
+
+
+  it('should throw if magic annotation is used by factory', function() {
+    module(function($provide) {
+      $provide.factory({
+        '$test': function($rootScope) { return function() {}; }
+      });
+    });
+    inject(function($injector) {
+      expect(function() {
+        $injector.invoke(['$test', function(test) {}]);
+      }).toThrowMinErr('$injector', 'strictdi');
+    });
+  });
+
+
+  it('should throw if factory does not return a value', function() {
+    module(function($provide) {
+      $provide.factory('$test', function() {});
+    });
+    expect(function() {
+      inject(function($test) {});
+    }).toThrowMinErr('$injector', 'undef');
+  });
+
+
+  it('should always use provider as `this` when invoking a factory', function() {
+    var called = false;
+    function factoryFn() {
+      called = true;
+      // jshint -W040
+      expect(typeof this.$get).toBe('function');
+      return this;
+      // jshint +W040
+    }
+    module(function($provide) {
+      $provide.factory('$test', factoryFn);
+    });
+    inject(function($test) {});
+    expect(called).toBe(true);
+  });
+
+  it('should set strictDi property to true on the injector instance', inject(function($injector) {
+    expect($injector.strictDi).toBe(true);
+  }));
+});
