@@ -1,3 +1,4 @@
+/* jshint scripturl: true */
 'use strict';
 
 describe('sanitizeUri', function() {
@@ -25,13 +26,11 @@ describe('sanitizeUri', function() {
   describe('img[src] sanitization', function() {
 
     it('should sanitize javascript: urls', function() {
-      /* jshint scripturl:true */
       testUrl = "javascript:doEvilStuff()";
       expect(sanitizeImg(testUrl)).toBe('unsafe:javascript:doEvilStuff()');
     });
 
     it('should sanitize javascript: urls with comments', function() {
-      /* jshint scripturl:true */
       testUrl = "javascript:alert(1)//data:image/";
       expect(sanitizeImg(testUrl)).toBe('unsafe:javascript:alert(1)//data:image/');
     });
@@ -44,20 +43,12 @@ describe('sanitizeUri', function() {
       expect(sanitizeImg(testUrl)).toBe("unsafe:data:,foo");
     });
 
-    it('should not sanitize data: URIs for images', function() {
-      // image data uri
-      // ref: http://probablyprogramming.com/2009/03/15/the-tiniest-gif-ever
-      testUrl = "data:image/gif;base64,R0lGODlhAQABAAAAACH5BAEKAAEALAAAAAABAAEAAAICTAEAOw==";
-      expect(sanitizeImg(testUrl)).toBe('data:image/gif;base64,R0lGODlhAQABAAAAACH5BAEKAAEALAAAAAABAAEAAAICTAEAOw==');
-    });
-
     it('should sanitize mailto: urls', function() {
       testUrl = "mailto:foo@bar.com";
       expect(sanitizeImg(testUrl)).toBe('unsafe:mailto:foo@bar.com');
     });
 
     it('should sanitize obfuscated javascript: urls', function() {
-      /* jshint scripturl:true */
       // case-sensitive
       testUrl = "JaVaScRiPt:doEvilStuff()";
       expect(sanitizeImg(testUrl)).toBe('unsafe:javascript:doEvilStuff()');
@@ -87,7 +78,6 @@ describe('sanitizeUri', function() {
     });
 
     it('should sanitize ng-src bindings as well', function() {
-      /* jshint scripturl:true */
       testUrl = "javascript:doEvilStuff()";
       expect(sanitizeImg(testUrl)).toBe('unsafe:javascript:doEvilStuff()');
     });
@@ -122,9 +112,19 @@ describe('sanitizeUri', function() {
       expect(sanitizeImg(testUrl)).toBe('file:///foo/bar.html');
     });
 
+    it('should not sanitize blob urls', function() {
+      testUrl = "blob:///foo/bar.html";
+      expect(sanitizeImg(testUrl)).toBe('blob:///foo/bar.html');
+    });
+
+    it('should not sanitize data: URIs for images', function() {
+      // image data uri
+      // ref: http://probablyprogramming.com/2009/03/15/the-tiniest-gif-ever
+      testUrl = "data:image/gif;base64,R0lGODlhAQABAAAAACH5BAEKAAEALAAAAAABAAEAAAICTAEAOw==";
+      expect(sanitizeImg(testUrl)).toBe('data:image/gif;base64,R0lGODlhAQABAAAAACH5BAEKAAEALAAAAAABAAEAAAICTAEAOw==');
+    });
 
     it('should allow reconfiguration of the src whitelist', function() {
-      /* jshint scripturl:true */
       var returnVal;
       expect(sanitizeUriProvider.imgSrcSanitizationWhitelist() instanceof RegExp).toBe(true);
       returnVal = sanitizeUriProvider.imgSrcSanitizationWhitelist(/javascript:/);
@@ -143,7 +143,6 @@ describe('sanitizeUri', function() {
   describe('a[href] sanitization', function() {
 
     it('should sanitize javascript: urls', inject(function() {
-      /* jshint scripturl:true */
       testUrl = "javascript:doEvilStuff()";
       expect(sanitizeHref(testUrl)).toBe('unsafe:javascript:doEvilStuff()');
     }));
@@ -156,7 +155,6 @@ describe('sanitizeUri', function() {
 
 
     it('should sanitize obfuscated javascript: urls', inject(function() {
-      /* jshint scripturl:true */
       // case-sensitive
       testUrl = "JaVaScRiPt:doEvilStuff()";
       expect(sanitizeHref(testUrl)).toBe('unsafe:javascript:doEvilStuff()');
@@ -187,7 +185,6 @@ describe('sanitizeUri', function() {
 
 
     it('should sanitize ngHref bindings as well', inject(function() {
-      /* jshint scripturl:true */
       testUrl = "javascript:doEvilStuff()";
       expect(sanitizeHref(testUrl)).toBe('unsafe:javascript:doEvilStuff()');
     }));
@@ -226,7 +223,6 @@ describe('sanitizeUri', function() {
     }));
 
     it('should allow reconfiguration of the href whitelist', function() {
-      /* jshint scripturl:true */
       var returnVal;
       expect(sanitizeUriProvider.aHrefSanitizationWhitelist() instanceof RegExp).toBe(true);
       returnVal = sanitizeUriProvider.aHrefSanitizationWhitelist(/javascript:/);
