@@ -2,12 +2,6 @@
 
 describe('SCE', function() {
 
-  // Work around an IE8 bug.  Though window.inject === angular.mock.inject, if it's invoked the
-  // window scope, IE8 loses the exception object that bubbles up and replaces it with a TypeError.
-  // By using a local alias, it gets invoked on the global scope instead of window.
-  // Ref: https://github.com/angular/angular.js/pull/4221#/issuecomment-25515813
-  var inject = angular.mock.inject;
-
   describe('when disabled', function() {
     beforeEach(function() {
       module(function($sceProvider) {
@@ -26,13 +20,21 @@ describe('SCE', function() {
     }));
   });
 
-  describe('IE8 quirks mode', function() {
+  describe('IE<11 quirks mode', function() {
+    /* global msie: true */
+    var msieBackup;
+
+    beforeEach(function() {
+      msieBackup = msie;
+    });
+
+    afterEach(function() {
+      msie = msieBackup;
+    });
+
     function runTest(enabled, documentMode, expectException) {
+      msie = documentMode;
       module(function($provide) {
-        $provide.value('$sniffer', {
-          msie: documentMode,
-          msieDocumentMode: documentMode
-        });
         $provide.value('$sceDelegate', {trustAs: null, valueOf: null, getTrusted: null});
       });
 
@@ -47,7 +49,7 @@ describe('SCE', function() {
         if (expectException) {
           expect(constructSce).toThrowMinErr(
             '$sce', 'iequirks', 'Strict Contextual Escaping does not support Internet Explorer ' +
-              'version < 9 in quirks mode.  You can fix this by adding the text <!doctype html> to ' +
+              'version < 11 in quirks mode.  You can fix this by adding the text <!doctype html> to ' +
               'the top of your HTML document.  See http://docs.angularjs.org/api/ng.$sce for more ' +
               'information.');
         } else {
@@ -120,11 +122,11 @@ describe('SCE', function() {
     }));
 
     it('should wrap undefined into undefined', inject(function($sce) {
-      expect($sce.trustAsHtml(undefined)).toBe(undefined);
+      expect($sce.trustAsHtml(undefined)).toBeUndefined();
     }));
 
     it('should unwrap undefined into undefined', inject(function($sce) {
-      expect($sce.getTrusted($sce.HTML, undefined)).toBe(undefined);
+      expect($sce.getTrusted($sce.HTML, undefined)).toBeUndefined();
     }));
 
     it('should wrap null into null', inject(function($sce) {
@@ -152,7 +154,7 @@ describe('SCE', function() {
     it('should NOT unwrap values when the type is different', inject(function($sce) {
       var originalValue = "originalValue";
       var wrappedValue = $sce.trustAs($sce.HTML, originalValue);
-      expect(function () { $sce.getTrusted($sce.CSS, wrappedValue); }).toThrowMinErr(
+      expect(function() { $sce.getTrusted($sce.CSS, wrappedValue); }).toThrowMinErr(
           '$sce', 'unsafe', 'Attempting to use an unsafe value in a safe context.');
     }));
 
@@ -196,7 +198,7 @@ describe('SCE', function() {
   });
 
 
-  describe('$sce.parseAs', function($sce) {
+  describe('$sce.parseAs', function() {
     it('should parse constant literals as trusted', inject(function($sce) {
       expect($sce.parseAsJs('1')()).toBe(1);
       expect($sce.parseAsJs('1', $sce.ANY)()).toBe(1);
@@ -205,10 +207,35 @@ describe('SCE', function() {
       expect($sce.parseAsJs('true')()).toBe(true);
       expect($sce.parseAsJs('false')()).toBe(false);
       expect($sce.parseAsJs('null')()).toBe(null);
-      expect($sce.parseAsJs('undefined')()).toBe(undefined);
+      expect($sce.parseAsJs('undefined')()).toBeUndefined();
       expect($sce.parseAsJs('"string"')()).toBe("string");
     }));
 
+    it('should be possible to do one-time binding', function() {
+      module(provideLog);
+      inject(function($sce, $rootScope, log) {
+        $rootScope.$watch($sce.parseAsHtml('::foo'), function(value) {
+          log(value + '');
+        });
+
+        $rootScope.$digest();
+        expect(log).toEqual('undefined'); // initial listener call
+        log.reset();
+
+        $rootScope.foo = $sce.trustAs($sce.HTML, 'trustedValue');
+        expect($rootScope.$$watchers.length).toBe(1);
+        $rootScope.$digest();
+
+        expect($rootScope.$$watchers.length).toBe(0);
+        expect(log).toEqual('trustedValue');
+        log.reset();
+
+        $rootScope.foo = $sce.trustAs($sce.HTML, 'anotherTrustedValue');
+        $rootScope.$digest();
+        expect(log).toEqual(''); // watcher no longer active
+      });
+    });
+
     it('should NOT parse constant non-literals', inject(function($sce) {
       // Until there's a real world use case for this, we're disallowing
       // constant non-literals.  See $SceParseProvider.
@@ -255,10 +282,10 @@ describe('SCE', function() {
     function runTest(cfg, testFn) {
       return function() {
         module(function($sceDelegateProvider) {
-          if (cfg.whiteList !== undefined) {
+          if (isDefined(cfg.whiteList)) {
             $sceDelegateProvider.resourceUrlWhitelist(cfg.whiteList);
           }
-          if (cfg.blackList !== undefined) {
+          if (isDefined(cfg.blackList)) {
             $sceDelegateProvider.resourceUrlBlacklist(cfg.blackList);
           }
         });
