@@ -1,7 +1,8 @@
 'use strict';
 
 describe('$route', function() {
-  var $httpBackend;
+  var $httpBackend,
+      element;
 
   beforeEach(module('ngRoute'));
 
@@ -18,6 +19,87 @@ describe('$route', function() {
     };
   }));
 
+  afterEach(function() {
+    dealoc(element);
+  });
+
+  it('should allow cancellation via $locationChangeStart via $routeChangeStart', function() {
+    module(function($routeProvider) {
+      $routeProvider.when('/Edit', {
+        id: 'edit', template: 'Some edit functionality'
+      });
+      $routeProvider.when('/Home', {
+        id: 'home'
+      });
+    });
+    module(provideLog);
+    inject(function($route, $location, $rootScope, $compile, log) {
+      $rootScope.$on('$routeChangeStart', function(event, next, current) {
+        if (next.id === 'home' && current.scope.unsavedChanges) {
+          event.preventDefault();
+        }
+      });
+      element = $compile('<div><div ng-view></div></div>')($rootScope);
+      $rootScope.$apply(function() {
+        $location.path('/Edit');
+      });
+      $rootScope.$on('$routeChangeSuccess', log.fn('routeChangeSuccess'));
+      $rootScope.$on('$locationChangeSuccess', log.fn('locationChangeSuccess'));
+
+      // aborted route change
+      $rootScope.$apply(function() {
+        $route.current.scope.unsavedChanges = true;
+      });
+      $rootScope.$apply(function() {
+        $location.path('/Home');
+      });
+      expect($route.current.id).toBe('edit');
+      expect($location.path()).toBe('/Edit');
+      expect(log).toEqual([]);
+
+      // successful route change
+      $rootScope.$apply(function() {
+        $route.current.scope.unsavedChanges = false;
+      });
+      $rootScope.$apply(function() {
+        $location.path('/Home');
+      });
+      expect($route.current.id).toBe('home');
+      expect($location.path()).toBe('/Home');
+      expect(log).toEqual(['locationChangeSuccess', 'routeChangeSuccess']);
+    });
+  });
+
+  it('should allow redirects while handling $routeChangeStart', function() {
+    module(function($routeProvider) {
+      $routeProvider.when('/some', {
+        id: 'some', template: 'Some functionality'
+      });
+      $routeProvider.when('/redirect', {
+        id: 'redirect'
+      });
+    });
+    module(provideLog);
+    inject(function($route, $location, $rootScope, $compile, log) {
+      $rootScope.$on('$routeChangeStart', function(event, next, current) {
+        if (next.id === 'some') {
+          $location.path('/redirect');
+        }
+      });
+      $compile('<div><div ng-view></div></div>')($rootScope);
+      $rootScope.$on('$routeChangeStart', log.fn('routeChangeStart'));
+      $rootScope.$on('$routeChangeError', log.fn('routeChangeError'));
+      $rootScope.$on('$routeChangeSuccess', log.fn('routeChangeSuccess'));
+      $rootScope.$apply(function() {
+        $location.path('/some');
+      });
+
+      expect($route.current.id).toBe('redirect');
+      expect($location.path()).toBe('/redirect');
+      expect(log).toEqual(['routeChangeStart', 'routeChangeStart', 'routeChangeSuccess']);
+    });
+  });
+
   it('should route and fire change event', function() {
     var log = '',
         lastRoute,
@@ -58,7 +140,7 @@ describe('$route', function() {
       $location.path('/NONE');
       $rootScope.$digest();
       expect(log).toEqual('before();after();');
-      expect($route.current).toEqual(null);
+      expect($route.current).toEqual(undefined);
     });
   });
 
@@ -116,7 +198,7 @@ describe('$route', function() {
       $location.path('/NONE');
       $rootScope.$digest();
       expect(log).toEqual('before();after();');
-      expect($route.current).toEqual(null);
+      expect($route.current).toEqual(undefined);
     });
   });
 
@@ -169,7 +251,7 @@ describe('$route', function() {
       $location.path('/BLANK');
       $rootScope.$digest();
       expect(log).toEqual('before();after();');
-      expect($route.current).toEqual(null);
+      expect($route.current).toEqual(undefined);
 
       log = '';
       $location.path('/Book2/Moby/one/two/Chapter/Intro').search('p=123');
@@ -181,10 +263,35 @@ describe('$route', function() {
       $location.path('/BOOK2/Moby/one/two/CHAPTER/Intro').search('p=123');
       $rootScope.$digest();
       expect(log).toEqual('before();after();');
-      expect($route.current).toEqual(null);
+      expect($route.current).toEqual(undefined);
     });
   });
 
+  it('should allow configuring caseInsensitiveMatch on the route provider level', function() {
+    module(function($routeProvider) {
+      $routeProvider.caseInsensitiveMatch = true;
+      $routeProvider.when('/Blank', {template: 'blank'});
+      $routeProvider.otherwise({template: 'other'});
+    });
+    inject(function($route, $location, $rootScope) {
+      $location.path('/bLaNk');
+      $rootScope.$digest();
+      expect($route.current.template).toBe('blank');
+    });
+  });
+
+  it('should allow overriding provider\'s caseInsensitiveMatch setting on the route level', function() {
+    module(function($routeProvider) {
+      $routeProvider.caseInsensitiveMatch = true;
+      $routeProvider.when('/Blank', {template: 'blank', caseInsensitiveMatch: false});
+      $routeProvider.otherwise({template: 'other'});
+    });
+    inject(function($route, $location, $rootScope) {
+      $location.path('/bLaNk');
+      $rootScope.$digest();
+      expect($route.current.template).toBe('other');
+    });
+  });
 
   it('should not change route when location is canceled', function() {
     module(function($routeProvider) {
@@ -196,7 +303,7 @@ describe('$route', function() {
         event.preventDefault();
       });
 
-      $rootScope.$on('$beforeRouteChange', function(event) {
+      $rootScope.$on('$routeChangeSuccess', function(event) {
         throw new Error('Should not get here');
       });
 
@@ -242,6 +349,21 @@ describe('$route', function() {
       $rootScope.$digest();
       expect($route.current).toBeDefined();
     }));
+
+    it("should use route params inherited from prototype chain", function() {
+      function BaseRoute() {}
+      BaseRoute.prototype.templateUrl = 'foo.html';
+
+      module(function($routeProvider) {
+        $routeProvider.when('/foo', new BaseRoute());
+      });
+
+      inject(function($route, $location, $rootScope) {
+        $location.path('/foo');
+        $rootScope.$digest();
+        expect($route.current.templateUrl).toBe('foo.html');
+      });
+    });
   });
 
 
@@ -281,7 +403,7 @@ describe('$route', function() {
       $rootScope.$on('$routeChangeStart', callback);
       $location.path('/test');
       $rootScope.$digest();
-      callback.reset();
+      callback.calls.reset();
 
       $location.search({any: true});
       $rootScope.$digest();
@@ -314,7 +436,7 @@ describe('$route', function() {
 
 
   it('should chain whens and otherwise', function() {
-    module(function($routeProvider){
+    module(function($routeProvider) {
       $routeProvider.when('/foo', {templateUrl: 'foo.html'}).
           otherwise({templateUrl: 'bar.html'}).
           when('/baz', {templateUrl: 'baz.html'});
@@ -380,7 +502,7 @@ describe('$route', function() {
     it('should handle unknown routes with "otherwise" route definition', function() {
       function NotFoundCtrl() {}
 
-      module(function($routeProvider){
+      module(function($routeProvider) {
         $routeProvider.when('/foo', {templateUrl: 'foo.html'});
         $routeProvider.otherwise({templateUrl: '404.html', controller: NotFoundCtrl});
       });
@@ -399,7 +521,7 @@ describe('$route', function() {
         expect($route.current.controller).toBe(NotFoundCtrl);
         expect(onChangeSpy).toHaveBeenCalled();
 
-        onChangeSpy.reset();
+        onChangeSpy.calls.reset();
         $location.path('/foo');
         $rootScope.$digest();
 
@@ -411,14 +533,14 @@ describe('$route', function() {
 
 
     it('should update $route.current and $route.next when default route is matched', function() {
-      module(function($routeProvider){
+      module(function($routeProvider) {
         $routeProvider.when('/foo', {templateUrl: 'foo.html'});
         $routeProvider.otherwise({templateUrl: '404.html'});
       });
 
       inject(function($route, $location, $rootScope) {
         var currentRoute, nextRoute,
-            onChangeSpy = jasmine.createSpy('onChange').andCallFake(function(e, next) {
+            onChangeSpy = jasmine.createSpy('onChange').and.callFake(function(e, next) {
           currentRoute = $route.current;
           nextRoute = next;
         });
@@ -438,7 +560,7 @@ describe('$route', function() {
         expect(nextRoute.templateUrl).toBe('404.html');
         expect($route.current.templateUrl).toBe('404.html');
         expect(onChangeSpy).toHaveBeenCalled();
-        onChangeSpy.reset();
+        onChangeSpy.calls.reset();
 
         // match regular route
         $location.path('/foo');
@@ -448,7 +570,7 @@ describe('$route', function() {
         expect(nextRoute.templateUrl).toBe('foo.html');
         expect($route.current.templateUrl).toEqual('foo.html');
         expect(onChangeSpy).toHaveBeenCalled();
-        onChangeSpy.reset();
+        onChangeSpy.calls.reset();
 
         // match otherwise route again
         $location.path('/anotherUnknownRoute');
@@ -460,11 +582,28 @@ describe('$route', function() {
         expect(onChangeSpy).toHaveBeenCalled();
       });
     });
+
+
+    it('should interpret a string as a redirect route', function() {
+      module(function($routeProvider) {
+        $routeProvider.when('/foo', {templateUrl: 'foo.html'});
+        $routeProvider.when('/baz', {templateUrl: 'baz.html'});
+        $routeProvider.otherwise('/foo');
+      });
+
+      inject(function($route, $location, $rootScope) {
+        $location.path('/unknownRoute');
+        $rootScope.$digest();
+
+        expect($location.path()).toBe('/foo');
+        expect($route.current.templateUrl).toBe('foo.html');
+      });
+    });
   });
 
 
   describe('events', function() {
-    it('should not fire $after/beforeRouteChange during bootstrap (if no route)', function() {
+    it('should not fire $routeChangeStart/Success during bootstrap (if no route)', function() {
       var routeChangeSpy = jasmine.createSpy('route change');
 
       module(function($routeProvider) {
@@ -481,6 +620,10 @@ describe('$route', function() {
         $location.path('/no-route-here');
         $rootScope.$digest();
         expect(routeChangeSpy).not.toHaveBeenCalled();
+
+        $location.path('/one');
+        $rootScope.$digest();
+        expect(routeChangeSpy).toHaveBeenCalled();
       });
     });
 
@@ -585,7 +728,7 @@ describe('$route', function() {
         $routeProvider.when('/foo', { templateUrl: 'http://example.com/foo.html' });
       });
 
-      inject(function ($route, $location, $rootScope) {
+      inject(function($route, $location, $rootScope) {
         $location.path('/foo');
         expect(function() {
           $rootScope.$digest();
@@ -600,7 +743,7 @@ describe('$route', function() {
         $sceDelegateProvider.resourceUrlWhitelist([/^http:\/\/example\.com\/foo\.html$/]);
       });
 
-      inject(function ($route, $location, $rootScope) {
+      inject(function($route, $location, $rootScope) {
         $httpBackend.whenGET('http://example.com/foo.html').respond('FOO BODY');
         $location.path('/foo');
         $rootScope.$digest();
@@ -671,35 +814,36 @@ describe('$route', function() {
     });
 
 
-    it('should drop in progress route change when new route change occurs and old fails', function() {
-      module(function($routeProvider) {
+    it('should throw an error when a template is not found', function() {
+      module(function($routeProvider, $exceptionHandlerProvider) {
+        $exceptionHandlerProvider.mode('log');
         $routeProvider.
           when('/r1', { templateUrl: 'r1.html' }).
-          when('/r2', { templateUrl: 'r2.html' });
+          when('/r2', { templateUrl: 'r2.html' }).
+          when('/r3', { templateUrl: 'r3.html' });
       });
 
-      inject(function($route, $httpBackend, $location, $rootScope) {
-        var log = '';
-        $rootScope.$on('$routeChangeError', function(e, next, last, error) {
-          log += '$failed(' + next.templateUrl + ', ' + error.status + ');';
-        });
-        $rootScope.$on('$routeChangeStart', function(e, next) { log += '$before(' + next.templateUrl + ');'; });
-        $rootScope.$on('$routeChangeSuccess', function(e, next) { log += '$after(' + next.templateUrl + ');'; });
-
+      inject(function($route, $httpBackend, $location, $rootScope, $exceptionHandler) {
         $httpBackend.expectGET('r1.html').respond(404, 'R1');
-        $httpBackend.expectGET('r2.html').respond('R2');
-
         $location.path('/r1');
         $rootScope.$digest();
-        expect(log).toBe('$before(r1.html);');
-
-        $location.path('/r2');
-        $rootScope.$digest();
-        expect(log).toBe('$before(r1.html);$before(r2.html);');
 
         $httpBackend.flush();
-        expect(log).toBe('$before(r1.html);$before(r2.html);$after(r2.html);');
-        expect(log).not.toContain('$after(r1.html);');
+        expect($exceptionHandler.errors.pop().message).toContain("[$compile:tpload] Failed to load template: r1.html");
+
+        $httpBackend.expectGET('r2.html').respond('');
+        $location.path('/r2');
+        $rootScope.$digest();
+
+        $httpBackend.flush();
+        expect($exceptionHandler.errors.length).toBe(0);
+
+        $httpBackend.expectGET('r3.html').respond('abc');
+        $location.path('/r3');
+        $rootScope.$digest();
+
+        $httpBackend.flush();
+        expect($exceptionHandler.errors.length).toBe(0);
       });
     });
 
@@ -727,7 +871,7 @@ describe('$route', function() {
 
 
   it('should match route with and without trailing slash', function() {
-    module(function($routeProvider){
+    module(function($routeProvider) {
       $routeProvider.when('/foo', {templateUrl: 'foo.html'});
       $routeProvider.when('/bar/', {templateUrl: 'bar.html'});
     });
@@ -777,14 +921,14 @@ describe('$route', function() {
         $rootScope.$digest();
         expect($location.path()).toBe('/foo');
         expect($route.current.templateUrl).toBe('foo.html');
-        expect(onChangeSpy.callCount).toBe(2);
+        expect(onChangeSpy).toHaveBeenCalledTimes(2);
 
-        onChangeSpy.reset();
+        onChangeSpy.calls.reset();
         $location.path('/baz');
         $rootScope.$digest();
         expect($location.path()).toBe('/bar');
         expect($route.current.templateUrl).toBe('bar.html');
-        expect(onChangeSpy.callCount).toBe(2);
+        expect(onChangeSpy).toHaveBeenCalledTimes(2);
       });
     });
 
@@ -830,6 +974,32 @@ describe('$route', function() {
     });
 
 
+    it('should properly process route params which are both eager and optional', function() {
+      module(function($routeProvider) {
+        $routeProvider.when('/foo/:param1*?/:param2', {templateUrl: 'foo.html'});
+      });
+
+      inject(function($location, $rootScope, $route) {
+        $location.path('/foo/bar1/bar2/bar3/baz');
+        $rootScope.$digest();
+
+        expect($location.path()).toEqual('/foo/bar1/bar2/bar3/baz');
+        expect($route.current.params.param1).toEqual('bar1/bar2/bar3');
+        expect($route.current.params.param2).toEqual('baz');
+        expect($route.current.templateUrl).toEqual('foo.html');
+
+        $location.path('/foo/baz');
+        $rootScope.$digest();
+
+        expect($location.path()).toEqual('/foo/baz');
+        expect($route.current.params.param1).toEqual(undefined);
+        expect($route.current.params.param2).toEqual('baz');
+        expect($route.current.templateUrl).toEqual('foo.html');
+
+      });
+    });
+
+
     it('should properly interpolate optional and eager route vars ' +
        'when redirecting from path with trailing slash', function() {
       module(function($routeProvider) {
@@ -861,7 +1031,7 @@ describe('$route', function() {
         return '/custom';
       }
 
-      module(function($routeProvider){
+      module(function($routeProvider) {
         $routeProvider.when('/bar/:id/:subid/:subsubid', {templateUrl: 'bar.html'});
         $routeProvider.when('/foo/:id', {redirectTo: customRedirectFn});
       });
@@ -881,14 +1051,14 @@ describe('$route', function() {
         $routeProvider.when('/foo/:id/:extra', {redirectTo: '/bar/:id'});
       });
       inject(function($browser, $route, $location, $rootScope) {
-        var $browserUrl = spyOnlyCallsWithArgs($browser, 'url').andCallThrough();
+        var $browserUrl = spyOnlyCallsWithArgs($browser, 'url').and.callThrough();
 
         $location.path('/foo/id3/eId');
         $rootScope.$digest();
 
         expect($location.path()).toEqual('/bar/id3');
-        expect($browserUrl.mostRecentCall.args)
-            .toEqual(['http://server/#/bar/id3?extra=eId', true]);
+        expect($browserUrl.calls.mostRecent().args)
+            .toEqual(['http://server/#/bar/id3?extra=eId', true, null]);
       });
     });
   });
@@ -908,7 +1078,7 @@ describe('$route', function() {
         $rootScope.$digest();
         expect(reloaded).toHaveBeenCalled();
         expect($routeParams).toEqual({});
-        reloaded.reset();
+        reloaded.calls.reset();
 
         // trigger reload
         $location.search({foo: 'bar'});
@@ -937,9 +1107,9 @@ describe('$route', function() {
         $location.path('/foo');
         $rootScope.$digest();
         expect(routeChange).toHaveBeenCalled();
-        expect(routeChange.callCount).toBe(2);
+        expect(routeChange).toHaveBeenCalledTimes(2);
         expect(routeUpdate).not.toHaveBeenCalled();
-        routeChange.reset();
+        routeChange.calls.reset();
 
         // don't trigger reload
         $location.search({foo: 'bar'});
@@ -966,14 +1136,14 @@ describe('$route', function() {
         $location.path('/foo/aaa');
         $rootScope.$digest();
         expect(routeChange).toHaveBeenCalled();
-        expect(routeChange.callCount).toBe(2);
-        routeChange.reset();
+        expect(routeChange).toHaveBeenCalledTimes(2);
+        routeChange.calls.reset();
 
         $location.path('/foo/bbb');
         $rootScope.$digest();
         expect(routeChange).toHaveBeenCalled();
-        expect(routeChange.callCount).toBe(2);
-        routeChange.reset();
+        expect(routeChange).toHaveBeenCalledTimes(2);
+        routeChange.calls.reset();
 
         $location.search({foo: 'bar'});
         $rootScope.$digest();
@@ -1002,18 +1172,18 @@ describe('$route', function() {
         $location.path('/foo');
         $rootScope.$digest();
         expect(routeParamsWatcher).toHaveBeenCalledWith({});
-        routeParamsWatcher.reset();
+        routeParamsWatcher.calls.reset();
 
         // trigger reload
         $location.search({foo: 'bar'});
         $rootScope.$digest();
         expect(routeParamsWatcher).toHaveBeenCalledWith({foo: 'bar'});
-        routeParamsWatcher.reset();
+        routeParamsWatcher.calls.reset();
 
         $location.path('/bar/123').search({});
         $rootScope.$digest();
         expect(routeParamsWatcher).toHaveBeenCalledWith({barId: '123'});
-        routeParamsWatcher.reset();
+        routeParamsWatcher.calls.reset();
 
         // don't trigger reload
         $location.search({foo: 'bar'});
@@ -1055,7 +1225,7 @@ describe('$route', function() {
         return 'foo.html';
       }
 
-      module(function($routeProvider){
+      module(function($routeProvider) {
         $routeProvider.when('/bar/:id/:subid/:subsubid', {templateUrl: 'bar.html'});
         $routeProvider.when('/foo/:id', {templateUrl: customTemplateUrlFn});
       });
@@ -1069,36 +1239,234 @@ describe('$route', function() {
       });
     });
 
-
     describe('reload', function() {
+      var $location;
+      var $log;
+      var $rootScope;
+      var $route;
+      var routeChangeStartSpy;
+      var routeChangeSuccessSpy;
 
-      it('should reload even if reloadOnSearch is false', function() {
-        var routeChangeSpy = jasmine.createSpy('route change');
-
-        module(function($routeProvider) {
-          $routeProvider.when('/bar/:barId', {controller: angular.noop, reloadOnSearch: false});
+      beforeEach(module(function($routeProvider) {
+        $routeProvider.when('/bar/:barId', {
+          template: '',
+          controller: controller,
+          reloadOnSearch: false
         });
 
-        inject(function($route, $location, $rootScope, $routeParams) {
-          $rootScope.$on('$routeChangeSuccess', routeChangeSpy);
+        function controller($log) {
+          $log.debug('initialized');
+        }
+      }));
+      beforeEach(inject(function($compile, _$location_, _$log_, _$rootScope_, _$route_) {
+        $location = _$location_;
+        $log = _$log_;
+        $rootScope = _$rootScope_;
+        $route = _$route_;
 
-          $location.path('/bar/123');
-          $rootScope.$digest();
-          expect($routeParams).toEqual({barId:'123'});
-          expect(routeChangeSpy).toHaveBeenCalledOnce();
-          routeChangeSpy.reset();
+        routeChangeStartSpy = jasmine.createSpy('routeChangeStart');
+        routeChangeSuccessSpy = jasmine.createSpy('routeChangeSuccess');
 
-          $location.path('/bar/123').search('a=b');
-          $rootScope.$digest();
-          expect($routeParams).toEqual({barId:'123', a:'b'});
-          expect(routeChangeSpy).not.toHaveBeenCalled();
+        $rootScope.$on('$routeChangeStart', routeChangeStartSpy);
+        $rootScope.$on('$routeChangeSuccess', routeChangeSuccessSpy);
 
-          $route.reload();
-          $rootScope.$digest();
-          expect($routeParams).toEqual({barId:'123', a:'b'});
-          expect(routeChangeSpy).toHaveBeenCalledOnce();
-        });
+        element = $compile('<div><div ng-view></div></div>')($rootScope);
+      }));
+
+      it('should reload the current route', function() {
+        $location.path('/bar/123');
+        $rootScope.$digest();
+        expect($location.path()).toBe('/bar/123');
+        expect(routeChangeStartSpy).toHaveBeenCalledOnce();
+        expect(routeChangeSuccessSpy).toHaveBeenCalledOnce();
+        expect($log.debug.logs).toEqual([['initialized']]);
+
+        routeChangeStartSpy.calls.reset();
+        routeChangeSuccessSpy.calls.reset();
+        $log.reset();
+
+        $route.reload();
+        $rootScope.$digest();
+        expect($location.path()).toBe('/bar/123');
+        expect(routeChangeStartSpy).toHaveBeenCalledOnce();
+        expect(routeChangeSuccessSpy).toHaveBeenCalledOnce();
+        expect($log.debug.logs).toEqual([['initialized']]);
+
+        $log.reset();
       });
+
+      it('should support preventing a route reload', function() {
+        $location.path('/bar/123');
+        $rootScope.$digest();
+        expect($location.path()).toBe('/bar/123');
+        expect(routeChangeStartSpy).toHaveBeenCalledOnce();
+        expect(routeChangeSuccessSpy).toHaveBeenCalledOnce();
+        expect($log.debug.logs).toEqual([['initialized']]);
+
+        routeChangeStartSpy.calls.reset();
+        routeChangeSuccessSpy.calls.reset();
+        $log.reset();
+
+        routeChangeStartSpy.and.callFake(function(evt) { evt.preventDefault(); });
+
+        $route.reload();
+        $rootScope.$digest();
+        expect($location.path()).toBe('/bar/123');
+        expect(routeChangeStartSpy).toHaveBeenCalledOnce();
+        expect(routeChangeSuccessSpy).not.toHaveBeenCalled();
+        expect($log.debug.logs).toEqual([]);
+      });
+
+      it('should reload even if reloadOnSearch is false', inject(function($routeParams) {
+        $location.path('/bar/123');
+        $rootScope.$digest();
+        expect($routeParams).toEqual({barId: '123'});
+        expect(routeChangeSuccessSpy).toHaveBeenCalledOnce();
+        expect($log.debug.logs).toEqual([['initialized']]);
+
+        routeChangeSuccessSpy.calls.reset();
+        $log.reset();
+
+        $location.search('a=b');
+        $rootScope.$digest();
+        expect($routeParams).toEqual({barId: '123', a: 'b'});
+        expect(routeChangeSuccessSpy).not.toHaveBeenCalled();
+        expect($log.debug.logs).toEqual([]);
+
+        $route.reload();
+        $rootScope.$digest();
+        expect($routeParams).toEqual({barId: '123', a: 'b'});
+        expect(routeChangeSuccessSpy).toHaveBeenCalledOnce();
+        expect($log.debug.logs).toEqual([['initialized']]);
+
+        $log.reset();
+      }));
     });
   });
+
+  describe('update', function() {
+    it('should support single-parameter route updating', function() {
+      var routeChangeSpy = jasmine.createSpy('route change');
+
+      module(function($routeProvider) {
+        $routeProvider.when('/bar/:barId', {controller: angular.noop});
+      });
+
+      inject(function($route, $routeParams, $location, $rootScope) {
+        $rootScope.$on('$routeChangeSuccess', routeChangeSpy);
+
+        $location.path('/bar/1');
+        $rootScope.$digest();
+        routeChangeSpy.calls.reset();
+
+        $route.updateParams({barId: '2'});
+        $rootScope.$digest();
+
+        expect($routeParams).toEqual({barId: '2'});
+        expect(routeChangeSpy).toHaveBeenCalledOnce();
+        expect($location.path()).toEqual('/bar/2');
+      });
+    });
+
+    it('should support total multi-parameter route updating', function() {
+      var routeChangeSpy = jasmine.createSpy('route change');
+
+      module(function($routeProvider) {
+        $routeProvider.when('/bar/:barId/:fooId/:spamId/:eggId', {controller: angular.noop});
+      });
+
+      inject(function($route, $routeParams, $location, $rootScope) {
+        $rootScope.$on('$routeChangeSuccess', routeChangeSpy);
+
+        $location.path('/bar/1/2/3/4');
+        $rootScope.$digest();
+        routeChangeSpy.calls.reset();
+
+        $route.updateParams({barId: '5', fooId: '6', spamId: '7', eggId: '8'});
+        $rootScope.$digest();
+
+        expect($routeParams).toEqual({barId: '5', fooId: '6', spamId: '7', eggId: '8'});
+        expect(routeChangeSpy).toHaveBeenCalledOnce();
+        expect($location.path()).toEqual('/bar/5/6/7/8');
+      });
+    });
+
+    it('should support partial multi-parameter route updating', function() {
+      var routeChangeSpy = jasmine.createSpy('route change');
+
+      module(function($routeProvider) {
+        $routeProvider.when('/bar/:barId/:fooId/:spamId/:eggId', {controller: angular.noop});
+      });
+
+      inject(function($route, $routeParams, $location, $rootScope) {
+        $rootScope.$on('$routeChangeSuccess', routeChangeSpy);
+
+        $location.path('/bar/1/2/3/4');
+        $rootScope.$digest();
+        routeChangeSpy.calls.reset();
+
+        $route.updateParams({barId: '5', fooId: '6'});
+        $rootScope.$digest();
+
+        expect($routeParams).toEqual({barId: '5', fooId: '6', spamId: '3', eggId: '4'});
+        expect(routeChangeSpy).toHaveBeenCalledOnce();
+        expect($location.path()).toEqual('/bar/5/6/3/4');
+      });
+    });
+
+
+    it('should update query params when new properties are not in path', function() {
+      var routeChangeSpy = jasmine.createSpy('route change');
+
+      module(function($routeProvider) {
+        $routeProvider.when('/bar/:barId/:fooId/:spamId/', {controller: angular.noop});
+      });
+
+      inject(function($route, $routeParams, $location, $rootScope) {
+        $rootScope.$on('$routeChangeSuccess', routeChangeSpy);
+
+        $location.path('/bar/1/2/3');
+        $location.search({initial: 'true'});
+        $rootScope.$digest();
+        routeChangeSpy.calls.reset();
+
+        $route.updateParams({barId: '5', fooId: '6', eggId: '4'});
+        $rootScope.$digest();
+
+        expect($routeParams).toEqual({barId: '5', fooId: '6', spamId: '3', eggId: '4', initial: 'true'});
+        expect(routeChangeSpy).toHaveBeenCalledOnce();
+        expect($location.path()).toEqual('/bar/5/6/3/');
+        expect($location.search()).toEqual({eggId: '4', initial: 'true'});
+      });
+    });
+
+    it('should not update query params when an optional property was previously not in path', function() {
+      var routeChangeSpy = jasmine.createSpy('route change');
+
+      module(function($routeProvider) {
+        $routeProvider.when('/bar/:barId/:fooId/:spamId/:eggId?', {controller: angular.noop});
+      });
+
+      inject(function($route, $routeParams, $location, $rootScope) {
+        $rootScope.$on('$routeChangeSuccess', routeChangeSpy);
+
+        $location.path('/bar/1/2/3');
+        $location.search({initial: 'true'});
+        $rootScope.$digest();
+        routeChangeSpy.calls.reset();
+
+        $route.updateParams({barId: '5', fooId: '6', eggId: '4'});
+        $rootScope.$digest();
+
+        expect($routeParams).toEqual({barId: '5', fooId: '6', spamId: '3', eggId: '4', initial: 'true'});
+        expect(routeChangeSpy).toHaveBeenCalledOnce();
+        expect($location.path()).toEqual('/bar/5/6/3/4');
+        expect($location.search()).toEqual({initial: 'true'});
+      });
+    });
+
+    it('should complain if called without an existing route', inject(function($route) {
+      expect(function() { $route.updateParams(); }).toThrowMinErr('ngRoute', 'norout');
+    }));
+  });
 });
