@@ -1,12 +1,14 @@
 'use strict';
 
 describe('HTML', function() {
+  var ua = window.navigator.userAgent;
+  var isChrome = /Chrome/.test(ua) && !/Edge/.test(ua);
 
   var expectHTML;
 
   beforeEach(module('ngSanitize'));
   beforeEach(function() {
-    expectHTML = function(html){
+    expectHTML = function(html) {
       var sanitize;
       inject(function($sanitize) {
         sanitize = $sanitize;
@@ -22,20 +24,23 @@ describe('HTML', function() {
     var handler, start, text, comment;
     beforeEach(function() {
       text = "";
+      start = null;
       handler = {
-        start: function(tag, attrs, unary){
+        start: function(tag, attrs) {
           start = {
             tag: tag,
-            attrs: attrs,
-            unary: unary
+            attrs: attrs
           };
           // Since different browsers handle newlines differently we trim
           // so that it is easier to write tests.
-          angular.forEach(attrs, function(value, key) {
+          for (var i = 0, ii = attrs.length; i < ii; i++) {
+            var keyValue = attrs[i];
+            var key = keyValue.key;
+            var value = keyValue.value;
             attrs[key] = value.replace(/^\s*/, '').replace(/\s*$/, '');
-          });
+          }
         },
-        chars: function(text_){
+        chars: function(text_) {
           text += text_;
         },
         end:function(tag) {
@@ -47,38 +52,14 @@ describe('HTML', function() {
       };
     });
 
-    it('should parse comments', function() {
+    it('should not parse comments', function() {
       htmlParser('<!--FOOBAR-->', handler);
-      expect(comment).toEqual('FOOBAR');
-    });
-
-    it('should throw an exception for invalid comments', function() {
-      var caught=false;
-      try {
-        htmlParser('<!-->', handler);
-      }
-      catch (ex) {
-        caught = true;
-        // expected an exception due to a bad parse
-      }
-      expect(caught).toBe(true);
-    });
-
-    it('double-dashes are not allowed in a comment', function() {
-      var caught=false;
-      try {
-        htmlParser('<!-- -- -->', handler);
-      }
-      catch (ex) {
-        caught = true;
-        // expected an exception due to a bad parse
-      }
-      expect(caught).toBe(true);
+      expect(comment).not.toBeDefined();
     });
 
     it('should parse basic format', function() {
       htmlParser('<tag attr="value">text</tag>', handler);
-      expect(start).toEqual({tag:'tag', attrs:{attr:'value'}, unary:false});
+      expect(start).toEqual({tag:'tag', attrs:{attr:'value'}});
       expect(text).toEqual('text');
     });
 
@@ -87,18 +68,6 @@ describe('HTML', function() {
         toBe('&lt;- text1 text2 &lt;1 text1 text2 &lt;{');
     });
 
-    it('should throw badparse if text content contains "<" followed by "/" without matching ">"', function() {
-      expect(function() {
-        htmlParser('foo </ bar', handler);
-      }).toThrowMinErr('$sanitize', 'badparse', 'The sanitizer was unable to parse the following block of html: </ bar');
-    });
-
-    it('should throw badparse if text content contains "<" followed by an ASCII letter without matching ">"', function() {
-      expect(function() {
-        htmlParser('foo <a bar', handler);
-      }).toThrowMinErr('$sanitize', 'badparse', 'The sanitizer was unable to parse the following block of html: <a bar');
-    });
-
     it('should accept tag delimiters such as "<" inside real tags', function() {
       // Assert that the < is part of the text node content, and not part of a tag name.
       htmlParser('<p> 10 < 100 </p>', handler);
@@ -107,25 +76,25 @@ describe('HTML', function() {
 
     it('should parse newlines in tags', function() {
       htmlParser('<tag\n attr="value"\n>text</\ntag\n>', handler);
-      expect(start).toEqual({tag:'tag', attrs:{attr:'value'}, unary:false});
+      expect(start).toEqual({tag:'tag', attrs:{attr:'value'}});
       expect(text).toEqual('text');
     });
 
     it('should parse newlines in attributes', function() {
       htmlParser('<tag attr="\nvalue\n">text</tag>', handler);
-      expect(start).toEqual({tag:'tag', attrs:{attr:'value'}, unary:false});
+      expect(start).toEqual({tag:'tag', attrs:{attr:'\nvalue\n'}});
       expect(text).toEqual('text');
     });
 
     it('should parse namespace', function() {
       htmlParser('<ns:t-a-g ns:a-t-t-r="\nvalue\n">text</ns:t-a-g>', handler);
-      expect(start).toEqual({tag:'ns:t-a-g', attrs:{'ns:a-t-t-r':'value'}, unary:false});
+      expect(start).toEqual({tag:'ns:t-a-g', attrs:{'ns:a-t-t-r':'\nvalue\n'}});
       expect(text).toEqual('text');
     });
 
     it('should parse empty value attribute of node', function() {
-      htmlParser('<OPTION selected value="">abc</OPTION>', handler);
-      expect(start).toEqual({tag:'option', attrs:{selected:'', value:''}, unary:false});
+      htmlParser('<test-foo selected value="">abc</test-foo>', handler);
+      expect(start).toEqual({tag:'test-foo', attrs:{selected:'', value:''}});
       expect(text).toEqual('abc');
     });
   });
@@ -133,11 +102,17 @@ describe('HTML', function() {
   // THESE TESTS ARE EXECUTED WITH COMPILED ANGULAR
   it('should echo html', function() {
     expectHTML('hello<b class="1\'23" align=\'""\'>world</b>.').
-       toEqual('hello<b class="1\'23" align="&#34;&#34;">world</b>.');
+       toBeOneOf('hello<b class="1\'23" align="&#34;&#34;">world</b>.',
+                 'hello<b align="&#34;&#34;" class="1\'23">world</b>.');
   });
 
   it('should remove script', function() {
-    expectHTML('a<SCRIPT>evil< / scrIpt >c.').toEqual('ac.');
+    expectHTML('a<SCRIPT>evil< / scrIpt >c.').toEqual('a');
+    expectHTML('a<SCRIPT>evil</scrIpt>c.').toEqual('ac.');
+  });
+
+  it('should remove script that has newline characters', function() {
+    expectHTML('a<SCRIPT\n>\n\revil\n\r</scrIpt\n >c.').toEqual('ac.');
   });
 
   it('should remove DOCTYPE header', function() {
@@ -160,12 +135,16 @@ describe('HTML', function() {
     expectHTML('a<STyle>evil</stYle>c.').toEqual('ac.');
   });
 
+  it('should remove style that has newline characters', function() {
+    expectHTML('a<STyle \n>\n\revil\n\r</stYle\n>c.').toEqual('ac.');
+  });
+
   it('should remove script and style', function() {
     expectHTML('a<STyle>evil<script></script></stYle>c.').toEqual('ac.');
   });
 
   it('should remove double nested script', function() {
-    expectHTML('a<SCRIPT>ev<script>evil</sCript>il</scrIpt>c.').toEqual('ac.');
+    expectHTML('a<SCRIPT>ev<script>evil</sCript>il</scrIpt>c.').toEqual('ailc.');
   });
 
   it('should remove unknown  names', function() {
@@ -174,10 +153,11 @@ describe('HTML', function() {
 
   it('should remove unsafe value', function() {
     expectHTML('<a href="javascript:alert()">').toEqual('<a></a>');
+    expectHTML('<img src="foo.gif" usemap="#foomap">').toEqual('<img src="foo.gif">');
   });
 
   it('should handle self closed elements', function() {
-    expectHTML('a<hr/>c').toEqual('a<hr/>c');
+    expectHTML('a<hr/>c').toEqual('a<hr>c');
   });
 
   it('should handle namespace', function() {
@@ -204,7 +184,8 @@ describe('HTML', function() {
 
   it('should ignore back slash as escape', function() {
     expectHTML('<img alt="xxx\\" title="><script>....">').
-      toEqual('<img alt="xxx\\" title="&gt;&lt;script&gt;...."/>');
+      toBeOneOf('<img alt="xxx\\" title="&gt;&lt;script&gt;....">',
+                '<img title="&gt;&lt;script&gt;...." alt="xxx\\">');
   });
 
   it('should ignore object attributes', function() {
@@ -238,6 +219,94 @@ describe('HTML', function() {
     expectHTML(false).toBe('false');
   });
 
+
+  it('should strip svg elements if not enabled via provider', function() {
+    expectHTML('<svg width="400px" height="150px" xmlns="http://www.w3.org/2000/svg"><circle cx="50" cy="50" r="40" stroke="black" stroke-width="3" fill="red"></svg>')
+      .toEqual('');
+  });
+
+  if (isChrome) {
+    it('should prevent mXSS attacks', function() {
+      expectHTML('<a href="&#x3000;javascript:alert(1)">CLICKME</a>').toBe('<a>CLICKME</a>');
+    });
+  }
+
+  it('should strip html comments', function() {
+    expectHTML('<!-- comment 1 --><p>text1<!-- comment 2 -->text2</p><!-- comment 3 -->')
+      .toEqual('<p>text1text2</p>');
+  });
+
+
+  describe('SVG support', function() {
+
+    beforeEach(module(function($sanitizeProvider) {
+      $sanitizeProvider.enableSvg(true);
+    }));
+
+
+    it('should accept SVG tags', function() {
+      expectHTML('<svg width="400px" height="150px" xmlns="http://www.w3.org/2000/svg"><circle cx="50" cy="50" r="40" stroke="black" stroke-width="3" fill="red"></svg>')
+        .toBeOneOf('<svg width="400px" height="150px" xmlns="http://www.w3.org/2000/svg"><circle cx="50" cy="50" r="40" stroke="black" stroke-width="3" fill="red"></circle></svg>',
+                   '<svg xmlns="http://www.w3.org/2000/svg" height="150px" width="400px"><circle fill="red" stroke-width="3" stroke="black" r="40" cy="50" cx="50"></circle></svg>',
+                   '<svg width="400px" height="150px" xmlns="http://www.w3.org/2000/svg"><circle fill="red" stroke="black" stroke-width="3" cx="50" cy="50" r="40"></circle></svg>',
+                   '<svg width="400px" height="150px" xmlns="http://www.w3.org/2000/svg"><circle FILL="red" STROKE="black" STROKE-WIDTH="3" cx="50" cy="50" r="40"></circle></svg>');
+    });
+
+    it('should not ignore white-listed svg camelCased attributes', function() {
+      expectHTML('<svg preserveAspectRatio="true"></svg>')
+        .toBeOneOf('<svg preserveAspectRatio="true"></svg>',
+                   '<svg preserveAspectRatio="true" xmlns="http://www.w3.org/2000/svg"></svg>');
+
+    });
+
+    it('should sanitize SVG xlink:href attribute values', function() {
+      expectHTML('<svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink"><a xlink:href="javascript:alert()"></a></svg>')
+        .toBeOneOf('<svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink"><a></a></svg>',
+                   '<svg xmlns:xlink="http://www.w3.org/1999/xlink" xmlns="http://www.w3.org/2000/svg"><a></a></svg>',
+                   '<svg xmlns="http://www.w3.org/2000/svg"><a xmlns:xlink="http://www.w3.org/1999/xlink"></a></svg>');
+
+      expectHTML('<svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink"><a xlink:href="https://example.com"></a></svg>')
+        .toBeOneOf('<svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink"><a xlink:href="https://example.com"></a></svg>',
+                   '<svg xmlns:xlink="http://www.w3.org/1999/xlink" xmlns="http://www.w3.org/2000/svg"><a xlink:href="https://example.com"></a></svg>',
+                   '<svg xmlns="http://www.w3.org/2000/svg"><a xlink:href="https://example.com" xmlns:xlink="http://www.w3.org/1999/xlink"></a></svg>',
+                   '<svg xmlns="http://www.w3.org/2000/svg"><a xmlns:xlink="http://www.w3.org/1999/xlink" xlink:href="https://example.com"></a></svg>');
+    });
+
+    it('should sanitize unknown namespaced SVG attributes', function() {
+      expectHTML('<svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink"><a xlink:foo="javascript:alert()"></a></svg>')
+        .toBeOneOf('<svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink"><a></a></svg>',
+                   '<svg xmlns:xlink="http://www.w3.org/1999/xlink" xmlns="http://www.w3.org/2000/svg"><a></a></svg>',
+                   '<svg xmlns="http://www.w3.org/2000/svg"><a></a></svg>');
+
+      expectHTML('<svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink"><a xlink:bar="https://example.com"></a></svg>')
+        .toBeOneOf('<svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink"><a></a></svg>',
+                   '<svg xmlns:xlink="http://www.w3.org/1999/xlink" xmlns="http://www.w3.org/2000/svg"><a></a></svg>',
+                   '<svg xmlns="http://www.w3.org/2000/svg"><a></a></svg>');
+    });
+
+    it('should not accept SVG animation tags', function() {
+      expectHTML('<svg xmlns:xlink="http://www.w3.org/1999/xlink"><a><text y="1em">Click me</text><animate attributeName="xlink:href" values="javascript:alert(1)"/></a></svg>')
+        .toBeOneOf('<svg xmlns:xlink="http://www.w3.org/1999/xlink"><a><text y="1em">Click me</text></a></svg>',
+                   '<svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink"><a><text y="1em">Click me</text></a></svg>',
+                   '<svg xmlns="http://www.w3.org/2000/svg"><a><text y="1em">Click me</text></a></svg>');
+
+      expectHTML('<svg><a xmlns:xlink="http://www.w3.org/1999/xlink" xlink:href="?"><circle r="400"></circle>' +
+        '<animate attributeName="xlink:href" begin="0" from="javascript:alert(1)" to="&" /></a></svg>')
+        .toBeOneOf('<svg><a xmlns:xlink="http://www.w3.org/1999/xlink" xlink:href="?"><circle r="400"></circle></a></svg>',
+                   '<svg><a xlink:href="?" xmlns:xlink="http://www.w3.org/1999/xlink"><circle r="400"></circle></a></svg>',
+                   '<svg xmlns="http://www.w3.org/2000/svg"><a xlink:href="?" xmlns:xlink="http://www.w3.org/1999/xlink"><circle r="400"></circle></a></svg>',
+                   '<svg xmlns="http://www.w3.org/2000/svg"><a xmlns:xlink="http://www.w3.org/1999/xlink" xlink:href="?"><circle r="400"></circle></a></svg>');
+    });
+
+    it('should not accept SVG `use` tags', function() {
+      expectHTML('<svg><use xlink:href="test.svg#xss" /></svg>')
+        .toBeOneOf('<svg></svg>',
+                   '<svg xmlns:xlink="http://www.w3.org/1999/xlink"></svg>',
+                   '<svg xmlns="http://www.w3.org/2000/svg"></svg>');
+    });
+  });
+
+
   describe('htmlSanitizerWriter', function() {
     /* global htmlSanitizeWriter: false */
     if (angular.isUndefined(window.htmlSanitizeWriter)) return;
@@ -246,7 +315,7 @@ describe('HTML', function() {
     beforeEach(function() {
       html = '';
       uriValidator = jasmine.createSpy('uriValidator');
-      writer = htmlSanitizeWriter({push:function(text){html+=text;}}, uriValidator);
+      writer = htmlSanitizeWriter({push:function(text) {html+=text;}}, uriValidator);
     });
 
     it('should write basic HTML', function() {
@@ -324,21 +393,21 @@ describe('HTML', function() {
       it('should call the uri validator', function() {
         writer.start('a', {href:'someUrl'}, false);
         expect(uriValidator).toHaveBeenCalledWith('someUrl', false);
-        uriValidator.reset();
+        uriValidator.calls.reset();
         writer.start('img', {src:'someImgUrl'}, false);
         expect(uriValidator).toHaveBeenCalledWith('someImgUrl', true);
-        uriValidator.reset();
+        uriValidator.calls.reset();
         writer.start('someTag', {src:'someNonUrl'}, false);
         expect(uriValidator).not.toHaveBeenCalled();
       });
 
       it('should drop non valid uri attributes', function() {
-        uriValidator.andReturn(false);
+        uriValidator.and.returnValue(false);
         writer.start('a', {href:'someUrl'}, false);
         expect(html).toEqual('<a>');
 
         html = '';
-        uriValidator.andReturn(true);
+        uriValidator.and.returnValue(true);
         writer.start('a', {href:'someUrl'}, false);
         expect(html).toEqual('<a href="someUrl">');
       });
@@ -347,22 +416,18 @@ describe('HTML', function() {
 
   describe('uri checking', function() {
     beforeEach(function() {
-      this.addMatchers({
+      jasmine.addMatchers({
         toBeValidUrl: function() {
-          var sanitize;
-          inject(function($sanitize) {
-            sanitize = $sanitize;
-          });
-          var input = '<a href="'+this.actual+'"></a>';
-          return sanitize(input) === input;
-        },
-        toBeValidImageSrc: function() {
-          var sanitize;
-          inject(function($sanitize) {
-            sanitize = $sanitize;
-          });
-          var input = '<img src="'+this.actual+'"/>';
-          return sanitize(input) === input;
+          return {
+            compare: function(actual) {
+              var sanitize;
+              inject(function($sanitize) {
+                sanitize = $sanitize;
+              });
+              var input = '<a href="' + actual + '"></a>';
+              return { pass: sanitize(input) === input };
+            }
+          };
         }
       });
     });
@@ -373,12 +438,12 @@ describe('HTML', function() {
         $provide.value('$$sanitizeUri', $$sanitizeUri);
       });
       inject(function() {
-        $$sanitizeUri.andReturn('someUri');
+        $$sanitizeUri.and.returnValue('someUri');
 
         expectHTML('<a href="someUri"></a>').toEqual('<a href="someUri"></a>');
         expect($$sanitizeUri).toHaveBeenCalledWith('someUri', false);
 
-        $$sanitizeUri.andReturn('unsafe:someUri');
+        $$sanitizeUri.and.returnValue('unsafe:someUri');
         expectHTML('<a href="someUri"></a>').toEqual('<a></a>');
       });
     });
@@ -389,13 +454,13 @@ describe('HTML', function() {
         $provide.value('$$sanitizeUri', $$sanitizeUri);
       });
       inject(function() {
-        $$sanitizeUri.andReturn('someUri');
+        $$sanitizeUri.and.returnValue('someUri');
 
-        expectHTML('<img src="someUri"/>').toEqual('<img src="someUri"/>');
+        expectHTML('<img src="someUri"/>').toEqual('<img src="someUri">');
         expect($$sanitizeUri).toHaveBeenCalledWith('someUri', true);
 
-        $$sanitizeUri.andReturn('unsafe:someUri');
-        expectHTML('<img src="someUri"/>').toEqual('<img/>');
+        $$sanitizeUri.and.returnValue('unsafe:someUri');
+        expectHTML('<img src="someUri"/>').toEqual('<img>');
       });
     });
 
@@ -466,14 +531,13 @@ describe('HTML', function() {
 });
 
 describe('decodeEntities', function() {
-  var handler, text,
-      origHiddenPre = window.hiddenPre;
+  var handler, text;
 
   beforeEach(function() {
     text = '';
     handler = {
       start: function() {},
-      chars: function(text_){
+      chars: function(text_) {
         text = text_;
       },
       end: function() {},
@@ -482,37 +546,13 @@ describe('decodeEntities', function() {
     module('ngSanitize');
   });
 
-  afterEach(function() {
-    window.hiddenPre = origHiddenPre;
+  it('should unescape text', function() {
+    htmlParser('a&lt;div&gt;&amp;&lt;/div&gt;c', handler);
+    expect(text).toEqual('a<div>&</div>c');
   });
 
-  it('should use innerText if textContent is not available (IE<9)', function() {
-    window.hiddenPre = {
-      innerText: 'INNER_TEXT'
-    };
-    inject(function($sanitize) {
-      htmlParser('<tag>text</tag>', handler);
-      expect(text).toEqual('INNER_TEXT');
-    });
-  });
-  it('should use textContent if available', function() {
-    window.hiddenPre = {
-      textContent: 'TEXT_CONTENT',
-      innerText: 'INNER_TEXT'
-    };
-    inject(function($sanitize) {
-      htmlParser('<tag>text</tag>', handler);
-      expect(text).toEqual('TEXT_CONTENT');
-    });
-  });
-  it('should use textContent even if empty', function() {
-    window.hiddenPre = {
-      textContent: '',
-      innerText: 'INNER_TEXT'
-    };
-    inject(function($sanitize) {
-      htmlParser('<tag>text</tag>', handler);
-      expect(text).toEqual('');
-    });
+  it('should preserve whitespace', function() {
+    htmlParser('  a&amp;b ', handler);
+    expect(text).toEqual('  a&b ');
   });
 });
